//==============================================================================
// PROJECT      = Bootloader
// MODULE       = bl78k0_kx2_control.c
// SHORT DESC.  = Bootloader control module
// DEVICE       = 78K0/Kx2
// VERSION      = 1.0
// DATE         = 05.12.2006
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================



#include "io78f0547_80.h"
#include "spl78k0_kx2.h"                // NEC EE
#include "spl78k0_types.h"              // NEC-EE
#include "spl78k0_kx2_user.h"           // NEC EE
#include "spl78k0_kx2_config.h"         // NEC EE
#include "spl78k0_kx2_const.h"          // NEC EE
#include "bl78k0_kx2_user.h"
#include "bl78k0_kx2_help_fkt.h"
#include "bl78k0_kx2_control.h"
#include "bl78k0_kx2_dbuffer.h"
#include "bl78k0_kx2_const.h"
#include "bl78k0_kx2_exceptions.h"
#include <intrinsics.h>





//*****************************************************************************
// extern data buffer for self. lib
//*****************************************************************************
extern __no_init u08  data_buffer[DATA_BUFFER_SIZE];


//*****************************************************************************
// extern functions: interface
//*****************************************************************************
extern void ifaceInit(void);
extern __callt u08 getRxByte(void);
extern __callt u08 byteReceived(void);
extern void sendByte(u08 txData);
extern void stopRx(void);
extern void readyToRx(void);
extern u08 txStatus(void);
extern u08 stopRxEnabled(void);
extern u08 readyToRxEnabled(void);
extern __callt u08 rxError(void);
extern void resetRxErrorFlags(void);
extern void resetInterface(void);


//*****************************************************************************
// extern functions: file_decoder
//*****************************************************************************
extern u08 asciiDecode(u08 ascii);
extern __callt u08 decodeReceivedBytes(u08 rx_byte);
extern u08 isEOF(void);
extern u32 getAddress(void);
extern u08 getWriteByte(void);
extern __callt void resetFileDecoder(void);

//*****************************************************************************
// extern functions: self lib
//*****************************************************************************
extern void  SelfLib_Open(void);
extern void  SelfLib_Close(void);


//*****************************************************************************
// extern functions: time module
//*****************************************************************************
extern void init_timer_50ms(void);
extern __callt void initTimeoutDetect(u08 timer_factor);
extern __callt void resetTimeoutDetect(void);
extern __callt u08 isTimeout(void);
extern __callt void setTimerIntervall_50ms(void);
extern __callt void setTimerIntervall_2ms(void);

//*****************************************************************************
// global variables
//*****************************************************************************
__saddr u08 temp_u08;                     // temporary 8 Bit variable
__saddr u16 temp_u16;                     // temporary 16 Bit variable
__saddr u32 temp_u32;                     // temporary 32 Bit variable

__saddr union union_addr union_u32;       // temporary 32 Bit union variable
__saddr union union_addr union_second_u32;// temporary 32 Bit union variable


u08 rx_buffer[RX_BUFFER_SIZE];            // receive buffer
__saddr u08 is_not_first_data;            // signalled first write into flash
__saddr u08 update_soft_or_bootl;         // bootl. or appl. update flag
                                          // (1: bootl 0:appl.)

__saddr u32 * pAddr;                      // temp 32 Bit pointer
__saddr u08 app_valid;                    // appl. valid bit
__saddr u08 exception_byte;               // current exception holder
__saddr u08 user_pressed_update;          // store users selection (bootl. or. appl update)
__saddr u08 temp_string[3];               // temporary char array for exception message



//*****************************************************************************
// Function:    resetAll
// Parameter:   None
// Return:      None
//
// Description: This function reset. all modules and essential variables!
//*****************************************************************************
__callt void resetAll(void)
{
  is_not_first_data = 0;                  // reset first write into flash signal
  update_soft_or_bootl = 0;               // reset
  app_valid = 1;                          // set app. is valid(for code optimisation)
  exception_byte = 0;                     // reset exeption byte holder
  user_pressed_update = 0;                // reset user update selection
  resetDataBufferControl();               // reset data buffer
  resetFileDecoder();                     // reset filedecoder
  resetInterface();                       // reset interface
  setTimerIntervall_50ms();               // set timer intervall on 50ms
}




//*****************************************************************************
// Function:    bootloader_start
// Parameter:   None
// Return:      u08 -> constants see within bl78k0_kx2_control.h
//      UPDATE:            start update
//      ERROR_OCCURS:      error occurs
//      APP_START:         start application
//
// Description: bootloader start messages and user prompt
//*****************************************************************************
u08 bootloader_start(void)
{

  #ifdef BOOTL_UPDATE_ALLOWED
    u08 temp = 0;                         // reset temporary variable
    u08 * flag_u08 = &temp_u08;           // set pointer for swap flag read
  #endif

  resetAll();                             // reset all modules of bootloader
  #ifdef BOOTL_UPDATE_ALLOWED

    SelfLib_Open();                       // open self. lib. environment

    if( SelfLib_ModeCheck() == STS_NO_ERROR ) // check voltage level on FLMD0
    {
      SelfLib_Init(data_buffer);          // initalization of data buffer

      temp = SelfLib_GetInfo_BootCluster( flag_u08 ); // is swap flag set?
      if( temp != STS_NO_ERROR )          // is error occured on swap flag read?
      {
        exception_byte = GET_INFO_ERROR;  // write error(0x16) id for exception handler
        exceptionHandler();               // performing exception handler
      }

      if( *flag_u08 == 1 )                // is swap flag set?
      {
        afterFirstSwap();                 // copy bootcluster 0 to bootcluster 1 and swap
      }
    }
    else                                  // low voltage level on FLMD0 pin
    {
      exception_byte = FLMD0_CHECK_ERROR; // write error(0x14) id for exception handler
      exceptionHandler();                 // performing exception handler

      while(1)                            // infinite loop
      {
        __no_operation();                 // NOP
      }
    }
  #endif

  u08 start_state = BOOTL_SET_RX_READY;   // set first bootloader state(interface receive-ready)

  while(1)
  {

    switch(start_state)                   // bootlaoder state?
    {

      case BOOTL_SET_RX_READY:            // set interface receive-ready

            if( readyToRxWithTimeout() )    // is interface don't  receive ready?
            {
              // timeout, interface don't ready
              exception_byte = STOP_RX_IFACE_ERROR;     // write error(0x02) id for exception handler
              return ERROR_OCCURS;                      // return error occurs
            }
            // receive a character

            if( !wait_for_rx(ILLEGAL_BYTE_TIMEOUT))     // wait on illegal byte
            {
                // timeout, no byte received
                if( !rxError() )                        // is error on receive interface?
                {
                  exception_byte = RECEIVE_ERROR;       // write error(0x05) id for exception handler
                  return ERROR_OCCURS;                  // return error occurs
                }

                start_state = BOOTL_START_MESSAGES;     // enter bootloader state -> show messages
                break;
            }
            else                                        // illegal byte recieved
            {
              sendString(ILLEGAL_CHARACTERS); // send message that the user stop to send on terminal
              start_state = BOOTL_ILLEGAL_CHARACTERS; // enter bootloader state -> rx other illegal bytes
              break;
            }

    case BOOTL_ILLEGAL_CHARACTERS:

      // if don't receive any byte within 700ms, then start bootloader (state: BOOTL_START_MESSAGES)
      // if interface receive one byte within 700ms, then restart timer and clear received byte.
      // wait again for 700ms!
        while(1)
        {

          if(!wait_for_rx(ILLEGAL_SEND_TIMEOUT))  // is timeout in receive?
          {
            resetRxErrorFlags();                  // reset error flags on receive interface

            start_state = BOOTL_START_MESSAGES;   // enter bootl. state -> send start messages
            break;
          }
        }

        break;

      case BOOTL_START_MESSAGES:                  // bootl. state -> send start messages

            sendString(VERSION_TEXT);             // is timeout on message send ("BV:")?
            sendString(BOOTLOADER_VERSION);       // is timeout on message send (version)?

            if( rom_Read(FIRST_ADDR_BCLUSTER1) != APP_VALID_PATTERN ) // is app. valid?
            {
              sendString( BAD_APP_CRC );          // send message application isn't valid
              app_valid = 0;                       // set bit, that application isn't valid
            }


            sendString(UPDATE_TEXT);              // send update message "Update?y or n" or "Update?a or b"
            start_state = BOOTL_RX_Y_OR_N;        // enter bootloader state -> user prompt "y or n"
        break;

      case BOOTL_RX_Y_OR_N:

        if(!wait_for_rx(RECEIVE_TIMEOUT_K_50MS))  // timeout, don't want to update??
        {
            // timeout
            if( !rxError() )                      // is error on rx interface?
            {
              exception_byte = RECEIVE_ERROR;     // write error(0x05) id for exception handler
              return ERROR_OCCURS;                // return error occurs
            }
            // user don't pressed any key, so the program can start
            if( app_valid )
              return APP_START;                     // start application
        }
        else                                      // user pressed any key within 2s
        {
          switch(getRxByte())                     // which key does the user press?
            {

              #ifdef BOOTL_UPDATE_ALLOWED
                case 'b':                           // user pressed 'b'
                  user_pressed_update = BOOTLOADER_UPDATE;  // bootloader update
                  return UPDATE;                    // start update
                case  'a':                          // user pressed 'n'
                  user_pressed_update = APPLICATION_UPDATE; // bootloader update
                  return UPDATE;                    // start application
              #else
                case 'y':                           // user pressed 'y'
                  return UPDATE;                    // start update
                case  'n':                          // user pressed 'n'
                  if( app_valid )
                  {
                    return APP_START;               // start application
                  }
                  else
                  {
                    start_state = BOOTL_SET_RX_READY; // retry user prompt
                    break;
                  }
              #endif

              default:                              // user pressed any key
                start_state = BOOTL_SET_RX_READY;   // retry user prompt
                break;
            }
        }

      default:
        break;
    }
  }
}



//*****************************************************************************
// Function:    executeUpdate
// Parameter:   None
// Return:      u08
//      ERROR_OCCURS:      error occurs
//      APP_START:         start application
//
// Description: This function perform control an update
//*****************************************************************************
u08 executeUpdate(void)
{

  u08 update_state = START_UPDATE;        // variable for update states
  u16 rx_count = 0;                       // byte receive count
  u08 timeout = 0;                        // timeout bit for EOF detection!
  #ifdef BOOTL_UPDATE_ALLOWED
    u16 * crc_pt;                         // pointer for crc read from flash
  #endif

  while(1)
  {
    switch(update_state)                  // which update state?
    {

      case START_UPDATE:                  // start update state

        #ifndef BOOTL_UPDATE_ALLOWED
          SelfLib_Open();                 // open self. lib. environment

          if( SelfLib_ModeCheck() == STS_NO_ERROR ) // check voltage level on FLMD0 pin
          {
            SelfLib_Init(data_buffer);    // init. data buffer
          }
          else                            // low voltage level on FLMD0 pin
          {
            exception_byte = FLMD0_CHECK_ERROR;     // write error(0x14) id for exception handler
            exceptionHandler();                     // performing exception handler

            while(1)                                // infinite loop
            {
              __no_operation();                     // NOP
            }
          }
        #endif

        sendString(SEND_FILE_TEXT);       // send message "send file"

        if( app_valid )                             // APP is valid?
        {
          if( !wait_for_rx(FIRST_BYTE_HEX_TIMEOUT) )  // wait for the first byte from hex-file
          {
            // timeout
            if( !rxError() )                        // error on receive interface?
            {
              exception_byte = RECEIVE_ERROR;       // write error(0x05) id for exception handler
              return ERROR_OCCURS;                  // return error occurs
            }

            sendString(HEX_FILE_TIMEOUT);           // send message hex file receive timeout
                                                    // timeout detection is in these case don't essential
            return APP_START;                       // return start application
          }
        }
        else
        {
          while(!byteReceived())                    // wait on receive
          {
            __no_operation();                       // NOP
          }

          if( !rxError() )                          // is error on receive interface ?
          {
            exception_byte = RECEIVE_ERROR; // write error(0x05) id for exception handler
            return ERROR_OCCURS;            // return error occurs
          }
        }

        rx_buffer[0] = getRxByte();         // fetch received byte
        rx_count++;                         // increment receive count


        update_state = RECEIVE_BYTES;       // enter receive state

        break;

      case RECEIVE_BYTES:                   // receive state

        setTimerIntervall_2ms();            // set timer on 2ms interval

        #if BYTES_AFTER_RX_STOP > 0

          temp_u16 = RX_BUFFER_SIZE - BYTES_AFTER_RX_STOP;  // set receive count without after XOFF bytes

          while(rx_count < temp_u16)                        // not all bytes received
          {
            if( !wait_for_rx(RX_BYTE_TIMEOUT_HEX_2MS) )     // wait for the first byte from hex-file
            {
              if( !rxError() )                              // is error on receive interface
              {
                exception_byte = RECEIVE_ERROR;             // write error(0x05) id for exception handler
                return ERROR_OCCURS;                        // return error occurs
              }
              timeout = 1;                                  // set timeout bit for EOF check
              break;
            }
            else
            {
              rx_buffer[rx_count] = getRxByte();            // fetch received byte
              rx_count++;                                   // increment receive count
            }
          }


          #ifndef STOP_RX_WITH_TIMEOUT
              stopRx();                             // hello interface, stop receive please
          #else
            if(stopRxWithTimeout())                           // XOFF is sent?
            {
              exception_byte = STOP_RX_IFACE_ERROR;           // bit set, interface not ready
              return ERROR_OCCURS;
            }
          #endif



          if( timeout == 1 )                                // is EOF timeout bit set?
          {
             update_state = ANALYSE_BYTES;                  // switch update state to anylse received bytes
             break;
          }

          temp_u08 = 0;                                     // reset temporary variable

          while( temp_u08 < BYTES_AFTER_RX_STOP )           // receive bytes after XOFF
          {
            if( !wait_for_rx(RX_BYTE_TIMEOUT_HEX_2MS) )     // wait for the first byte from hex-file
            {
              // timeout
              if( !rxError() )                              // is error on receive interface?
              {
                exception_byte = RECEIVE_ERROR;             // write error(0x05) id for exception handler
                return ERROR_OCCURS;                        // return error occurs
              }
              break;
            }


            rx_buffer[rx_count] = getRxByte();              // fetch received byte
            rx_count++;                                     // increment receive count
            temp_u08++;                                     // increment
          }

          setTimerIntervall_50ms();                         // set intervall timer on 50ms

        #else

          while(rx_count < RX_BUFFER_SIZE)                  // not all bytes received?
          {
            if( !wait_for_rx(RX_BYTE_TIMEOUT_HEX_2MS) )     // wait for the first byte from hex-file
            {
              if( !rxError() )                              // is error on receive interface?
              {
                exception_byte = RECEIVE_ERROR;             // write error(0x05) id for exception handler
                return ERROR_OCCURS;                        // return error occurs
              }
              timeout = 1;                                  // set EOF timeout bit
              break;
            }

          rx_buffer[rx_count] = getRxByte();                // fetch received byte
          rx_count++;                                       // increment received count
          }

          setTimerIntervall_50ms();                         // set timer on 50ms

          if(stopRxWithTimeout())                           // is interface receive ready
          {
            exception_byte = STOP_RX_IFACE_ERROR;           // write error(0x02) id for exception handler
            return ERROR_OCCURS;                            // return error occurs
          }
        #endif


        update_state = ANALYSE_BYTES;                       // switch update state to anylse received bytes
        break;



      case ANALYSE_BYTES:       // here we can assume that receive_count bytes are written

       for(u16 z = 0; z < rx_count; z++ )                   // are all bytes analysed?
       {
          temp_u08 = decodeReceivedBytes( rx_buffer[z] );  // analyse next byte

          if( temp_u08 == 1 )                               // is error on hex file?
          {
            exception_byte = HEX_FILE_ERROR;                // write error(0x03) id for exception handler
            return ERROR_OCCURS;                            // return error occurs
          }
          else if(temp_u08 == 3)                            // write in data buffer?
          {
            if( !is_not_first_data )                        // is first data to write?
            {
              is_not_first_data = 1;                        // first data processed

              if( !eraseAppArea() )                         // application area erased complete without error?
              {
                exception_byte = APP_ERASE_ERROR;           // write error(0x13) id for exception handler
                return ERROR_OCCURS ;                       // return error occurs
              }

              temp_u32 = getAddress();                      // get the first write Address

              #ifdef BOOTL_UPDATE_ALLOWED

                if( temp_u32 < (u32)FIRST_ADDR_BCLUSTER1 )  // is first address in the bootloader area?
                {
                  if( user_pressed_update == APPLICATION_UPDATE)// user want to update appl.?
                  {
                    exception_byte = HEX_FILE_IS_IN_BOOTL_AREA; // set exception id
                    return ERROR_OCCURS;                        // return error occurs
                  }

                  update_soft_or_bootl = 1;                     // bootloader update is set
                }
                else  // first address is in the application area
                {

                  if( user_pressed_update == BOOTLOADER_UPDATE) // user want to update bootloader?
                  {
                    exception_byte = HEX_FILE_IS_IN_APP_AREA;   // set exception id
                    return ERROR_OCCURS;                        // return error occurs
                  }

                  update_soft_or_bootl = 0;                     // application update
                }

              #else
                if( temp_u32 < (u32)FIRST_ADDR_BCLUSTER1 )    // is first address in the bootloader area?
                {
                  // bootloader update don't allowed
                  exception_byte = BOOTL_UPDATE_NOT_ALLOWED;  // write error(0x19) id for exception handler
                  return ERROR_OCCURS;                        // return error occurs
                }
              #endif


            }

            while(1)                                          // wait until all bytes are written into the dbuffer
            {
                setByteAddr(getAddress());                    // set flash write address for the first byte in data buffer

                temp_u08 = dB_write(getWriteByte());          // write byte in data buffer


                if( temp_u08 == 1 )                           // is error occured on data buffer write?
                {
                  exception_byte = DATA_BUFFER_WRITE_ERROR;   // write error(0x07) id for exception handler
                  return ERROR_OCCURS;                        // return error occurs

                }
                else if( temp_u08 == 2 )                      // data to write into the flash?
                {
                   if( !writeIntoFlash() )                    // is error occured on flash write?
                    {
                      exception_byte = FLASH_WRITE_ERROR;     // write error(0x12) id for exception handler
                      return ERROR_OCCURS;                    // return error occurs
                    }
                }
                else
                {
                  break;
                }

              } // while

          }
          else                            // get next byte to analyse
          {
            __no_operation();             // NOP
          }

        }   // end of the for loop

        rx_count = 0;                     // reset rx_count, all bytes analysed and written

          if( timeout == 1 )              // is timeout occured?
          {
            if( !isEOF() )                // isn't EOF?
            {
              exception_byte = FILE_TRANSFER_TIMEOUT;   // write error(0x04) id for exception handler
              return ERROR_OCCURS ;                     // return error occurs
            }

            if( isLastDataToWrite() )                   // is any data to write into the flash?
            {
              if( !writeIntoFlash() )                   // write data into flash, is error on write?
              {
                exception_byte = FLASH_WRITE_ERROR;     // write error(0x12) id for exception handler
                return ERROR_OCCURS;                    // return error occurs
              }
            }
            update_state = AFTER_FLASH_WRITE;           // enter after flash write state
            break;

          }
          else                                          // no timeout on receive!
          {

            if(readyToRxWithTimeout())                  // is interface ready to rx?
            {
              exception_byte = STOP_RX_IFACE_ERROR;     // interface don't ready
              return ERROR_OCCURS;                      // return error occurs
            }

            update_state = RECEIVE_BYTES;               // set update state on receive bytes
            break;
          }

          // state: do something after flash write
      case AFTER_FLASH_WRITE:

        sendString( HEX_FILE_RECEIVED );                // send message " File OK!"

        if( !verifyAppArea())                           // is everything ok on app. area verify?
        {
          exception_byte = VERIFY_BYTES_ERROR;          // set exception(0x11) error
          return ERROR_OCCURS;                          // return error occurs
        }

        #ifdef BOOTL_UPDATE_ALLOWED
        if( update_soft_or_bootl )                      // is bootloader update?
        {
          calcBootlSec1CRC();                           // chalculate CRC over bootsector 1
          temp_u16 = BL_CRC_ADDRESS + FIRST_ADDR_BCLUSTER1; // set pointer on CRC within the ROM area
          crc_pt = (u16 *)temp_u16;                     // read CRC

          if( *crc_pt != getCRC())                      // is calculated CRC equal to the read CRC
          {
            exception_byte = BOOTL_UPDATE_CRC_ERROR;    // set exception(0x09) error
            return ERROR_OCCURS;                        // return error occurs
          }

          temp_u08 = SelfLib_SetInfo_SwapBootCluster(); // execute swap with the SelfLib

          if (temp_u08 != STS_NO_ERROR)                 // is error on swapping?
          {
            exception_byte = SWAP_SET_INFO_ERROR;       // set exception(0x10) error
            return ERROR_OCCURS;                        // return error occurs
          }

          START_WATCHDOG;                               // start watchdog

          while(1)                                      // infinity loop
          {
            __no_operation();                           // NOP
          }

        }
        else  // Software update
        {
          if( !checkAppCRC() )                          // calc. Application CRC
          {
            exception_byte = APP_UPDATE_CRC_ERROR;      // set exception(0x08) error
            return ERROR_OCCURS;                        // return error occurs
          }

          #if DATA_BUFFER_SIZE < 4
            #error DATA_BUFFER_SIZE is smaller than 4
          #endif

          data_buffer[0] = APP_VALID_PATTERN;           // write appl. is valid
          data_buffer[1] = 0;
          data_buffer[2] = 0;
          data_buffer[3] = 0;

          // write one word into the flash
          temp_u08 =  SelfLib_EepWrite( FIRST_ADDR_BCLUSTER1, 1);

          if (temp_u08 != STS_NO_ERROR)                 // is error occured on flash write?
          {
            exception_byte = FLASH_WRITE_ERROR;         // set exception(0x12) error
            return ERROR_OCCURS;                        // return error occurs
          }

          if( !verifyAppArea() )                        // is an error occured on appl. area verify?
          {
            exception_byte = VERIFY_BYTES_ERROR;        // set exception(0x11) error
            return ERROR_OCCURS;                        // return error occurs
          }

          sendString( UPDATE_READY );                   // send message "Update is Ready"
          return APP_START;                             // return -> application start
        }

        #else
          if( !checkAppCRC() )                          // is appl. CRC ok?
          {
            exception_byte = APP_UPDATE_CRC_ERROR;      // set exception(0x08) error
            return ERROR_OCCURS;                        // return error occurs
          }

          #if DATA_BUFFER_SIZE < 4
            #error DATA_BUFFER_SIZE is smaller than 4
          #endif

          data_buffer[0] = APP_VALID_PATTERN;           // write appl. is valid pattern
          data_buffer[1] = 0;
          data_buffer[2] = 0;
          data_buffer[3] = 0;

          // write one word into the flash
          temp_u08 =  SelfLib_EepWrite( FIRST_ADDR_BCLUSTER1, 1);

          if (temp_u08 != STS_NO_ERROR)                 // is error on sweapping?
          {
            exception_byte = FLASH_WRITE_ERROR;         // set exception(0x12) error
            return ERROR_OCCURS;                        // return error occurs
          }

          if( !verifyAppArea() )                        // is an error occured on appl. area verify?
          {
            exception_byte = VERIFY_BYTES_ERROR;        // set exception(0x11) error
            return ERROR_OCCURS;                        // return error occurs
          }

          sendString( UPDATE_READY );                   // send message "Update is Ready"
          return APP_START;                             // Start application

        #endif

      default:
        break;

    }
  }

}



//*****************************************************************************
// Function:    writeIntoFlash
// Parameter:   None
// Return:      u08
//      ERROR:  error occurs
//      OK:     all bytes ar written into the flash
//
// Description: This function communicate with the DataBuffer module.
//              Written bytes in the data buffer will be written into the flash.
//*****************************************************************************

u08 writeIntoFlash(void)
{
  union_u32.u32_access = getFirstWriteAddress();    // get first flash write addr.
  temp_u08 = getWordCount();                        // get word count to write

  #ifdef LAST_APP_BLOCK
    if(getBlockNr() > LAST_APP_BLOCK )              // is block nr. over the boundary?
    {
      exception_byte = DEF_BLOCK_END_EXCEEDED;      // set exception(0x17) error
      return ERROR ;                                // return -> error occurs
    }
  #endif

  #ifdef BOOTL_UPDATE_ALLOWED
  if( update_soft_or_bootl )                        // is bootloader update?
    {
      // is addr. over bootcluster boundary?
      if( union_u32.u16_access.U16_1 > FIRST_ADDR_BCLUSTER1 )
      {
        exception_byte = BOOTL_UPDATE_ADDR_ERROR;   // set exception(0x18) error
        return ERROR;                               // return error
      }

      // add offset of 4K for bootl. update -> bootl. must be in the bootsector 1
      union_u32.u16_access.U16_1 += FIRST_ADDR_BCLUSTER1;
    }
  #endif

  // write into the flash!
  temp_u08 =  SelfLib_EepWrite( union_u32.u32_access, temp_u08);
  if (temp_u08 != STS_NO_ERROR)                     // is error on flash write?
  {
    exception_byte = FLASH_WRITE_ERROR;             // set exception(0x12) error
    return ERROR;                                   // return error occurs
  }

  return OK;                                        // return -> all bytes written

}





//*****************************************************************************
// Function:    afterFirstSwap
// Parameter:   None
// Return:      None
//
// Description: This function will be used, if the first swap is processed.
//
//*****************************************************************************
void afterFirstSwap(void)
{
    u08 self_lib_result = 0;
    u16 crc_temp = 0;
    u16 * crc_pt;

  if( !eraseAppArea())                              // error on erase?
  {
    exception_byte = APP_ERASE_ERROR;               // set exception(0x13) error
    exceptionHandler();                             // perform exception handler

    while(1)                                        // infinity loop!
    {
      __no_operation();                             // NOP
    }
  }
  else                                              // no error on app. erase!
  {
    if( !copyBootCluster() )                        // error on copy bootcluster 0 to 1
    {
      exception_byte = COPY_BCLUSTER_ERROR;         // set exception(0x15) error
      exceptionHandler();                           // perform exception handler
      while(1)                                      // infinity loop
      {
        __no_operation();                           // NOP
      }
    }
    else                                            // no error on bootcluster copy
    {
      if( !verifyAppArea())                         // error on app. area verify?
      {
        exception_byte = VERIFY_BYTES_ERROR;        // set exception(0x11) error
        exceptionHandler();                         // perform exception handler

        while(1)                                    // infinity loop
        {
          __no_operation();                         // NOP
        }
    }

    calcBootlSec1CRC();                             // calculate CRC of bootcluster(bootl. 1) 1
    crc_pt = (u16 *)BL_CRC_ADDRESS;                 // read CRC of the bootcluster 0
    crc_temp = getCRC();

    if( *crc_pt != crc_temp )                       // is CRC of bootcluster 0 and 1 the same?
    {
      exception_byte = BOOTL_UPDATE_CRC_ERROR;      // set exception(0x09) error
      exceptionHandler();                           // perform exception handler

      while(1)                                      // infinity loop
      {
        __no_operation();                           // NOP
      }
    }


    #if DATA_BUFFER_SIZE < 4
      #error DATA_BUFFER_SIZE is smaller than 4
    #endif


    data_buffer[0] = 0;
    data_buffer[1] = 0;
    data_buffer[2] = (u08)(crc_temp & 0x00FF);      // write CRC 8 Bit LSB
    data_buffer[3] = (u08)(crc_temp >> 8);          // write CRC 8 Bit MSB

    // write bootloader CRC into the flash of the bootcluster 1
    self_lib_result =  SelfLib_EepWrite( CRC_BL_HEADER + FIRST_ADDR_BCLUSTER1, 1);

    if ( self_lib_result != STS_NO_ERROR )          // is error on write?
    {
      exception_byte = FLASH_WRITE_ERROR;           // set exception(0x12) error
      exceptionHandler();                           // perform exception handler

      START_WATCHDOG;                               // start watchdog

      while(1)                                      // infinity loop
      {
        __no_operation();                           // NOP
      }
    }

    // set swap flag in the hidden rom area
    self_lib_result = SelfLib_SetInfo_SwapBootCluster();

    if (self_lib_result != STS_NO_ERROR)            // is error on swapflag set?
    {
      exception_byte = SWAP_SET_INFO_ERROR;         // set exception(0x10) error
      exceptionHandler();                           // perform exception handler

      while(1)                                      // infinity loop
      {
        __no_operation();                           // NOP
      }
    }

    sendString( UPDATE_READY );                     // send message "Update ready"
    START_WATCHDOG;                                 // start watchdog

    while(1)                                        // infinity loop <- reset generating
    {
      __no_operation();                             // NOP
    }
  }

 }
}




//*****************************************************************************
// Function:    calcBootlSec1CRC
// Parameter:   None
// Return:      None
//
// Description: Calculate CRC over the bootcluster 1 area
//              Start and End of the area to calculate must be always the same
//*****************************************************************************
__callt void calcBootlSec1CRC(void)
{
  u16 end_addr;
  u08 rom_read_byte;

  resetCRC();                                       // reset old CRC

  end_addr = LAST_BOOTL_CODE_ADDR + FIRST_ADDR_BCLUSTER1; // set calc. end addr.

  for( u16 i = FIRST_ADDR_BCLUSTER1; i <= end_addr; i++ ) // all bytes included in calc.
  {
    rom_read_byte = rom_Read(i);                    // read byte from bootcl. 1 area
    calcCRC( rom_read_byte );                       // calc CRC
  }

  // The calcCRC must be called with 2 zeros after calculation!
  calcCRC(0x00);
  calcCRC(0x00);
}



//*****************************************************************************
// Function:    checkAppCRC
// Parameter:   None
// Return:      u08
//              OK :    calc. and read CRC are equal
//              ERROR:  calc. CRC isn't equal to read CRC or addr. error
//
// Description: Calculate CRC over the application area
//              The start and end addresses are defined in the application area
//              on addresses APP_START_ADR and APP_END_ADR
//*****************************************************************************
__callt u08 checkAppCRC(void)
{

  u16 * crc_pt;
  u16 read_crc;
  u08 rom_read_byte;

  resetCRC();                             // reset old CRC

  pAddr = (u32 *)APP_START_ADR;           // set pointer on the start addr. location of the appl.
  union_u32.u32_access = *pAddr;          // read and save start address of the application

  pAddr = (u32 *)APP_END_ADR;             // set pointer on the end addr. location of the appl.
  union_second_u32.u32_access = *pAddr;   // read and save end address of the application

  #ifdef CODEBANK_BANKS_USER

  if( union_u32.u32_access > union_second_u32.u32_access )  // is end address smaller than start addr.
  {
    return ERROR;                             // return error occurs!
  }

  if( union_second_u32.u08_access.U08_3 > 0 ) // is end address outside the common area?
  {
    while( union_u32.u32_access <= union_second_u32.u32_access )  // all addresses read?
    {
      if( union_u32.u08_access.U08_3 >= CODEBANK_BANKS_USER )     // isn't valid bank nr.?
      {
        return ERROR;                                             // error!
      }

      BANK = union_u32.u08_access.U08_3;                          // select bank

      rom_read_byte = rom_Read(union_u32.u16_access.U16_1);       // read byte from flash addr.
      calcCRC( rom_read_byte );                                   // calc. CRC for read byte


      if( (u16)union_u32.u16_access.U16_1 < (u16)CODEBANK_END_USER )  // isn't boundary reached?
      {
        union_u32.u16_access.U16_1++;                                 // increment address
      }
      else                                                            // boundary reached!
      {
        // set addr. pointer on the first address in the bank
        union_u32.u16_access.U16_1 = CODEBANK_START_USER;
        union_u32.u08_access.U08_3++;                             // inc. address
      }

    }
  }
  else      // last address is in the common area!
  {
    // isn't last addr. valid( exceeds BANK0 )
    if( union_second_u32.u16_access.U16_1 > CODEBANK_END_USER )
    {
      return ERROR; // ERROR: Address exceeds BANK 0
    }

    // all bytes read?
    while( union_u32.u16_access.U16_1 <= union_second_u32.u16_access.U16_1 )
    {

      rom_read_byte = rom_Read(union_u32.u16_access.U16_1);       // read byte from flash!
      calcCRC( rom_read_byte );                                   // calc CRC for read byte!
      union_u32.u16_access.U16_1++;                               // set pointer on the next address
    }
  }

  #else

    #ifndef FLASH_END_ADDR
      #error FLASH_END_ADDR is not defined! Please define the end address of the micro.
    #endif

    if( union_second_u32.u08_access.U08_3 > 0 ) // is end address outside the common area?
    {
      return ERROR;                             // return error occurs!
    }

    if( union_u32.u16_access.U16_1 > union_second_u32.u16_access.U16_1 )  // is end address smaller than start addr.
    {
      return ERROR;                             // return error occurs!
    }

    if( union_second_u32.u16_access.U16_1 > FLASH_END_ADDR )  // is end address bigger than mem. area
    {
      return ERROR;                             // return error occurs!
    }

    while( union_u32.u16_access.U16_1 <= union_second_u32.u16_access.U16_1 )
    {

      rom_read_byte = rom_Read(union_u32.u16_access.U16_1);       // read byte from flash!
      calcCRC( rom_read_byte );                                   // calc CRC for read byte!
      union_u32.u16_access.U16_1++;                               // set pointer on the next address
    }

  #endif

  // The calcCRC must be called with 2 zeros after calculation!
  calcCRC(0x00);
  calcCRC(0x00);

  crc_pt = (u16 *)CRC_APP_LOCATION;                     // set CRC-Read-Pointer on CRC-Location

  read_crc = *crc_pt;                                   // read and save CRC

  if( read_crc == getCRC() )                            // calc. and read CRC are same?
  {
   return OK;                                           // appl. is OK!
  }

  return ERROR;                                         // error on CRC check
}


//*****************************************************************************
// Function:    verifyAppArea
// Parameter:   None
// Return:      u08
//              OK :    appl. area verified, anything is ok
//              ERROR:  error occured on internal verify
//
// Description: This function verify the application area, where the
//              data was written.
//*****************************************************************************
__callt u08 verifyAppArea(void)
{
  u08 block_num = FIRST_APP_BLOCK;                      // set iterator on first block
  u08 temp_result = 0;

  while(1)
  {
    #ifdef LAST_APP_BLOCK
      #if LAST_APP_BLOCK < FIRST_APP_BLOCK
        #error ERROR: defined LAST_APP_BLOCK is smaller then FIRST_APP_BLOCK
      #endif

      #if LAST_APP_BLOCK < 8
        #error ERROR: LAST_APP_BLOCK must be bigger then 7
      #endif

        if( block_num > LAST_APP_BLOCK )                // is last defined "LAST_APP_BLOCK" block reached?
        {
          return OK;                                    // return OK
        }
    #endif

    temp_result = SelfLib_Verify( block_num );          // verify block
    if( temp_result == STS_PARAMETER_ERROR )            // is parameter error on verify?
    {
      return OK;                                        // all blocks verified
    }
    else if( temp_result == STS_NO_ERROR )              // is no error?
    {
      __no_operation();                                 // do nothing and verify the next block
    }
    else                                                // not allowed interrupt or something else
    {
      return ERROR;                                     // return error occured
    }

    block_num++;                                        // increment block iterator
  }
}





//*****************************************************************************
// Function:    eraseAppArea
// Parameter:   None
// Return:      u08
//              OK :    appl. area erased, anything is ok
//              ERROR:  error occured on erase application area
//
// Description: This function erase the application area.  Before one block will
//              be erased , it check whether the block is blank.
//
//*****************************************************************************
__callt u08 eraseAppArea(void)
{
    u08 temp = FIRST_APP_BLOCK;
    u08 temp_blankcheck = 0;
    u08 temp_erase = 0;

    while( 1 )
    {
      #ifdef LAST_APP_BLOCK
        #if LAST_APP_BLOCK < FIRST_APP_BLOCK
          #error ERROR: defined LAST_APP_BLOCK is smaller then FIRST_APP_BLOCK
        #endif

        #if LAST_APP_BLOCK < 8
          #error ERROR: LAST_APP_BLOCK must be bigger then 7
        #endif

        if( temp > LAST_APP_BLOCK )                 // is last block reached?
        {
          return OK;                                // all blocks erased!
        }
      #endif

      temp_blankcheck = SelfLib_BlankCheck(temp);   // is block blank?

      if( temp_blankcheck == STS_NO_ERROR )         // no error on blank check?
      {
        temp++;                                     // next block
      }
      else if( temp_blankcheck == STS_MRG11_ERROR ) // block not blank
      {
        temp_erase = SelfLib_Erase(temp);           // erase block

        if( temp_erase != STS_NO_ERROR )            // is error on erase?
        {
          return ERROR;                             // return error occurs
        }

      }
      else if( temp_blankcheck == STS_PARAMETER_ERROR ) // is wrong block nr?
      {

         return OK;                                 // all block erased
      }
      else                                          // interrupted by user interrupt
      {
          return ERROR; // Interrupted -> ERROR because all interrupts must be disabled
      }
    }
}




//*****************************************************************************
// Function:    copyBootCluster
// Parameter:   None
// Return:      u08
//              OK :    bootcluster copy 0 to 1 was successful
//              ERROR:  error occured on write
//
// Description: This function copy the bootcluster 0 content to bootcluster 1
//
//*****************************************************************************
u08 copyBootCluster(void)
{
  u16 addr = 0;                           // set address iterator to 0
  u08 self_lib_result = 0;


  while( addr < LAST_BOOTL_CODE_ADDR )    // all data copied?
  {

      temp_u32 = (u32)addr + FIRST_ADDR_BCLUSTER1;  // add offset to addr. iterator

      for( u08 i = 0; i < 4; i++ )        // 4 byte read from bootcluster 0?
      {
        temp_u08 = rom_Read(addr);        // read byte from bootcluster 0
        data_buffer[i] = temp_u08;        // write byte into the data buffer
        addr++;                           // increment addr. iterator
      }

      // write data buffer content to the flash ( bootcluster 1)
      self_lib_result =  SelfLib_EepWrite( temp_u32, 0x01);

      if (self_lib_result != STS_NO_ERROR)// is error on flash write?
      {
        return ERROR;                     // return error is occured on flash write
      }

  }

  return OK;             // return bootcluster copy was successful
}




//*****************************************************************************
// Function:    readyToRxWithTimeout
// Parameter:   none
// Return:      u08
//              0: interface inform that it is ready for receive
//              1: timout is occured
// Description: This function signaled the transmitter, that it is receive ready!
//*****************************************************************************

__callt u08 readyToRxWithTimeout(void)
{
  initTimeoutDetect(READY_TO_RX_TIMEOUT);           // init. timeout detection
  readyToRx();                  // send XON character!
  while(!readyToRxEnabled())    // wait until the character was send!
  {
    if(isTimeout())             // timeout??
    {
      resetTimeoutDetect();     // reset timeout detection!
      return 1;                 // timeout occurs;
    }
  }
  resetTimeoutDetect();         // reset timeout detection!
  return 0;                     // timeout is occured
}






//*****************************************************************************
// Function:    stopRxWithTimeout
// Parameter:   none
// Return:      u08
//              0: interface stopped receive
//              1: timout is occured
// Description: This function signaled the transmitter, that it must stop send
//*****************************************************************************

__callt u08 stopRxWithTimeout(void)
{
  initTimeoutDetect(STOP_RX_TIMEOUT);   // init. timeout detection
  stopRx();                             // hello interface, stop receive please

  while(!stopRxEnabled())           // wait with timeout detection, whether interface is ready
  {
    if(isTimeout())                 // timeout??
    {
      resetTimeoutDetect();         // reset timeout detection
      return 1;                     // timeout
    }
  }
  resetTimeoutDetect();             // reset timeout detection
  return 0;                         // stopped
}



//*****************************************************************************
// Function:    sendString
// Parameter:   None
// Return:      u08
//                    0: timeout
//                    1: message send
// Description: With this function you can send Strings to the interface
//              If a timeout occur then the interface couldn`t send a String
//*****************************************************************************

__callt void sendString (u08 *ucpStr)
{
    u08 ucData;
    u08 time_out = 0;
    initTimeoutDetect(MESSAGE_TIMEOUT_K_50MS);  // init. timeout detection

    ucData = *ucpStr;                   // save character to send
    while(ucData)                       // isn't string end?
    {

      sendByte (ucData);                // send character
      while(txStatus())                 // tx ready???
      {
        if(isTimeout())                 // is timeout?
        {
          time_out = 1;                 // set timeout bit
          messageTimeoutHandler();      // call timeout message handler
          break;
        }
      }

      if( time_out )                    // is timeout?
        break;                          // leave message send loop

      ucpStr++;                         // send pointer on next character
      ucData = *ucpStr;                 // save character

    }
    resetTimeoutDetect();               // reset timeout detection
}



//*****************************************************************************
// Function:    messageTimeoutHandler
// Parameter:   None
// Return:      None
// Description: This function will be called, if timeout occured on message send!
//              For example:  If the Terminal can't communicate with micro. you
//                            can turn on one led!
//*****************************************************************************
void messageTimeoutHandler(void)
{
  __no_operation();                       // NOP
}




//*****************************************************************************
// Function:    wait_for_rx
// Parameter:   None
// Return:      u08
//                  1: received byte
//                  0: timeout
//
// Description: At this function the receiver wait for a byte with defined timeout
//              timeout = oflow * 50ms
//*****************************************************************************
__callt u08 wait_for_rx(u08 timeout_factor)
{
  initTimeoutDetect(timeout_factor);      // init timeout detection

  while(!byteReceived())                  // wait for interface receive byte
  {
    if(isTimeout())                       // overflow?
    {
      resetTimeoutDetect();               // reset timeout detection
      return 0;                           // overflow;
    }
  }
  resetTimeoutDetect();                   // reset timeout detection
  return 1;                               // return byte received
}


//*****************************************************************************
// Function:    wait_for_rx
// Parameter:   None
// Return:      None
//
// Description: This exception handler will be called, if an error occurs.
//*****************************************************************************
__callt void exceptionHandler(void)
{
  setTimerIntervall_50ms();               // set timer intervall on 50ms

  // while(txStatus());                      // is interface busy?
  sendString("\n\n\rERROR:");             // send Error Message
  temp_string[0] = getASCII( exception_byte >> 4 );   // send MSB nibble
  temp_string[1] = getASCII( exception_byte & 0x0F ); // send LSB nibble
  temp_string[2] = 0x00;                              // set String end
  sendString(temp_string);                // send String

  SelfLib_Close();                        // close Self Library

}




//*****************************************************************************
// Function:    init_micro
// Parameter:   None
// Return:      None
//
// Description: This function init. essential modules and disable interrupts!
//*****************************************************************************
void init_micro(void)
{
  __disable_interrupt();// disable all interrupts

  MOC_bit.no7 = 1;      // must be disabled to change the EXCLK and OSCSEL bits
  OSCCTL = 0xF1;        // external clock input mode
  MOC_bit.no7 = 0;      // enable external input
  PCC = 0x00;           // CPU clock = fxp
  MCM = 0x07;           // Set main system clock and peripheral hardware clock

  IMS = IMS_REGISTER_VALUE; // set IMS the internal memory size switching register
  IXS = IXS_REGISTER_VALUE; // set IXS internal expansion RAM size switching register


  OSTS = 0x00;          // Set minimum oscillation stabilization time
  RCM_bit.no0 = 1;      // Stop internal high-speed oscillator

  ifaceInit();          // init the interface
  init_timer_50ms();    // init Timer50 for timeout detection

}



